#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "odicon.h"

static void help(int argc, char *argv[]) {
    printf("usage: %s opt dim [seed]\n", argv[0]);
    printf("opt: M  = M convex minimize\n"
           "     L  = L convex minimize\n"
           "     S  = submodular minimize\n"
           "     QL = quasi L convex minimize\n"
           "     LV = L convex validate\n"
           "     MV = M convex validate\n"
           "     2  = quadratic\n");
}

int main(int argc, char *argv[]) {
    int dim, seed = 12345;

    if (argc <= 1) {
        help(argc, argv);
        return EXIT_FAILURE;
    }

    if (argc <= 2) {
        fprintf(stderr, "Give me dimension.\n");
        return EXIT_SUCCESS;
    }

    dim = atoi(argv[2]);
    if (argc > 3) seed = atoi(argv[3]);

    mt_initialize(seed);
    if (argv[1][0] != 'D') printf("%s: dim = %d, seed = %d\n", argv[0], dim, seed);
    cpu_time();
#define HILO(x,y) ((x)<<8|(y))

    switch (HILO(toupper(argv[1][0]),toupper(argv[1][1]))) {
    case HILO('M',0): test_mconv(dim);  break;
    case HILO('L',0): test_lconv2(dim); break;
    case HILO('S',0): test_submodular(dim); break;
    case HILO('Q','L'): test_qlconv(dim); break;
    case HILO('L','V'): test_lconv_v(dim); break;
    case HILO('M','V'): test_mconv_v(dim); break;
    case HILO('2',0):   test_quadratic(dim); break;
    case HILO('D','L'): /* demo */
    case HILO('D','M'):
    case HILO('D','Z'):
        cpu_time_limit = 5; /* 5sec */
        switch (argv[1][1]) {
        case 'L': demo_Lseparable(dim); break;
        case 'M': demo_Mseparable(dim); break;
        case 'Z': demo_procurement(dim); break;
        }
        break;
    default: help(argc, argv);
    }
    return EXIT_SUCCESS;
}
